/*--------------------------------------------------------------------------*/
#include "dd.h"
#include <ddraw.h>
/*--------------------------------------------------------------------------*/
int                 g_iBpp;     // Remember the main surface bit depth
HWND                g_hWnd;     // To store the main windows handle
LPDIRECTDRAW        g_pDD;      // The DirectDraw object
LPDIRECTDRAWCLIPPER g_pClipper; // Clipper for primary surface
LPDIRECTDRAWSURFACE g_pDDS;     // Primary surface
/*--------------------------------------------------------------------------*/
// Initialize DirectDraw stuff
int DDInit()
{
   HRESULT hr;
   
   g_hWnd = NULL;
   
   // FIXME: Enumerate
   hr = DirectDrawCreate( NULL, &g_pDD, NULL );
   if (FAILED(hr))
      return -1;
   
   return 0;
}
/*--------------------------------------------------------------------------*/
// Clean up DirectDraw stuff
void DDDone()
{
   if (g_pDD != NULL)
      g_pDD->Release();
}
/*--------------------------------------------------------------------------*/
// Create and set up the primary surface
int DDCreateVisual( HWND hWnd )
{
   DDSURFACEDESC ddsd;
   HRESULT hr;
   
   g_hWnd = hWnd;
   
   // Set the directdraw cooperative level
   hr = g_pDD->SetCooperativeLevel( g_hWnd, DDSCL_NORMAL );
   // FIXME: SHould destroy the window here etc.
   if (FAILED(hr))
      return -1;
   
   // Create the primary surface
   ddsd.dwSize = sizeof(ddsd);
   ddsd.dwFlags = DDSD_CAPS;
   ddsd.ddsCaps.dwCaps = DDSCAPS_PRIMARYSURFACE;
   hr = g_pDD->CreateSurface( &ddsd, &g_pDDS, NULL );
   if (FAILED(hr))
      return -2;
   
   // Create a clipper
   hr = g_pDD->CreateClipper( 0, &g_pClipper, NULL );
   if (FAILED(hr))
      return -3;
   hr = g_pClipper->SetHWnd( 0, g_hWnd );
   if (FAILED(hr))
      return -4;
   hr = g_pDDS->SetClipper( g_pClipper );
   if (FAILED(hr))
      return -5;
   
   // Lock surface to retreive surface information
   hr= g_pDDS->Lock( NULL, &ddsd, DDLOCK_WAIT, NULL );
   if (FAILED(hr))
      return -6;
   
   // Store bit depth of surface
   g_iBpp = ddsd.ddpfPixelFormat.dwRGBBitCount;
   
   // Unlock surface
   hr = g_pDDS->Unlock( NULL );
   if (FAILED(hr))
      return -7;
   
   return 0;
}
/*--------------------------------------------------------------------------*/
// PutPixel routine for a DirectDraw surface
void DDPutPixel( LPDIRECTDRAWSURFACE pDDS, int x, int y, int r, int g, int b )
{
   HRESULT hr;
   DDBLTFX ddbfx;
   RECT    rcDest;
   POINT   p;
   
   // Safety net
   if (g_pDDS == NULL)
      return;
   
   // Initialize the DDBLTFX structure with the pixel color
   ddbfx.dwSize = sizeof( ddbfx );
   ddbfx.dwFillColor = (DWORD)CreateRGB( r, g, b );
   
   // Prepare the destination rectangle as a 1x1 (1 pixel) rectangle
   p.x = x;
   p.y = y;
   ClientToScreen( g_hWnd, &p );
   OffsetRect( &rcDest, p.x, p.y );
   SetRect( &rcDest, p.x, p.y, p.x+1, p.y+1 );
   
   // Blit 1x1 rectangle using solid color op
   hr = g_pDDS->Blt( &rcDest, NULL, NULL, DDBLT_WAIT | DDBLT_COLORFILL, &ddbfx );
   if (FAILED(hr))
      OutputDebugString( (LPCTSTR)"Royal fuckup\n" );
}
/*--------------------------------------------------------------------------*/
// Create color from RGB triple
unsigned int CreateRGB( int r, int g, int b )
{
   unsigned int pixel;
   
   switch (g_iBpp)
   {
   case 8:
      // Here you should do a palette lookup to find the closes match.
      // I'm not going to bother with that. Many modern games no
      // longer support 256-color modes.
      break;
   case 16:
      // Break down r,g,b into 5-6-5 format.
      pixel = ((r/8)<<11) | ((g/4)<<5) | (b/8);
      break;
   case 24:
   case 32:
      pixel = (r<<16) | (g<<8) | (b);
      break;
   default:
      pixel =0;
   }
   
   return pixel;
}
/*--------------------------------------------------------------------------*/
